@@ -7,6 +7,7 @@ package org.jboss.as.jpa.config;
 
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Set;
 
 import jakarta.persistence.EntityManagerFactory;
 
@@ -186,7 +187,7 @@ public class Configuration {
     /**
      * name of the Hibernate Search module providing the outbox-polling coordination strategy
      */
-    public static final String HIBERNATE_SEARCH_MODULE_MAPPER_ORM_COORDINATION_OUTBOXPOLLING = "org.hibernate.search.mapper.orm.coordination.outboxpolling";
+    public static final String HIBERNATE_SEARCH_MODULE_MAPPER_ORM_OUTBOXPOLLING = "org.hibernate.search.mapper.orm.outboxpolling";
 
     /**
      * name of the Hibernate Search module providing the Lucene backend
@@ -227,6 +228,7 @@ public class Configuration {
     // key = provider class name, value = module name
     private static final Map<String, String> providerClassToModuleName = new HashMap<String, String>();
     private static final String HIBERNATE = "Hibernate";
+    private static final String HIBERNATE_ENHANCER = "hibernate.enhancer";
 
     static {
         // always choose the default hibernate version for the Hibernate provider class mapping
@@ -271,10 +273,30 @@ public class Configuration {
             return Boolean.parseBoolean(pu.getProperties().getProperty(Configuration.JPA_CONTAINER_CLASS_TRANSFORMER));
         }
         if (isHibernateProvider(pu.getPersistenceProviderClassName())) {
-            return false;
+
+            return isAnyHibernateEnhancerPropertySpecified(pu);
         }
         return true;
     }
+
+    /**
+     * Consider jboss.as.jpa.classtransformer to be set to true if any hibernate.enhancer properties are set to true.
+     *
+     * @param pu
+     * @return true if any hibernate.enhancer property is detected otherwise return false
+     */
+    private static boolean isAnyHibernateEnhancerPropertySpecified(PersistenceUnitMetadata pu) {
+
+        Set<String> set = pu.getProperties().stringPropertyNames();
+        for (String key : set) {
+            if (key.startsWith(HIBERNATE_ENHANCER) &&
+                    (true == Boolean.parseBoolean(pu.getProperties().getProperty(key)))) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     private static boolean isHibernateProvider(String provider) {
         // If the persistence provider is not specified (null case), Hibernate ORM will be used as the persistence provider.
         return provider == null || provider.contains(HIBERNATE);
