@@ -1,23 +1,6 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2011, Red Hat, Inc., and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 
 package org.jboss.as.jpa.messages;
@@ -26,6 +9,8 @@ import static org.jboss.logging.Logger.Level.ERROR;
 import static org.jboss.logging.Logger.Level.INFO;
 import static org.jboss.logging.Logger.Level.WARN;
 
+import java.lang.invoke.MethodHandles;
+
 import jakarta.ejb.EJBException;
 import jakarta.persistence.EntityManager;
 import jakarta.persistence.PersistenceException;
@@ -53,7 +38,7 @@ public interface JpaLogger extends BasicLogger {
     /**
      * Default root level logger with the package name for he category.
      */
-    JpaLogger ROOT_LOGGER = Logger.getMessageLogger(JpaLogger.class, "org.jboss.as.jpa");
+    JpaLogger ROOT_LOGGER = Logger.getMessageLogger(MethodHandles.lookup(), JpaLogger.class, "org.jboss.as.jpa");
 
     /**
      * Logs a warning message indicating duplicate persistence.xml files were found.
@@ -481,8 +466,8 @@ public interface JpaLogger extends BasicLogger {
      *
      * @return a {@link RuntimeException} for the error.
      */
-    @Message(id = 41, value = "Can only inject from a Hibernate EntityManagerFactoryImpl")
-    RuntimeException hibernateOnlyEntityManagerFactory();
+//     @Message(id = 41, value = "Can only inject from a Hibernate EntityManagerFactoryImpl")
+//     RuntimeException hibernateOnlyEntityManagerFactory();
 
 //    /**
 //     * Creates an exception indicating the entity manager factory implementation can only be a Hibernate version.
@@ -753,8 +738,8 @@ public interface JpaLogger extends BasicLogger {
 
     // id = 72, value = "Could not obtain TransactionListenerRegistry from transaction manager")
 
-    @Message(id = 73, value = "Transformation of class %s failed")
-    IllegalStateException invalidClassFormat(@Cause Exception cause, String className);
+    @Message(id = 73, value = "Bytecode rewrite (transformation) of class %s failed")
+    String invalidClassFormat(String className);
 
     // @Message(id = 74, value = "Deprecated Hibernate51CompatibilityTransformer is enabled for all application deployments.")
 
@@ -769,4 +754,7 @@ public interface JpaLogger extends BasicLogger {
     @Message(id = 74, value = "Persistence provider integrator module load error for %s")
     DeploymentUnitProcessingException cannotLoadPersistenceProviderIntegratorModule(@Cause Throwable cause, String persistenceProviderModule);
 
+    @Message(id = 75, value="Illegal to call this method from injected, managed EntityManager")
+    IllegalStateException illegalCallOnCloseMethod();
+
 }
