@@ -1,24 +1,7 @@
 /*
-* JBoss, Home of Professional Open Source.
-* Copyright 2012, Red Hat Middleware LLC, and individual contributors
-* as indicated by the @author tags. See the copyright.txt file in the
-* distribution for a full listing of individual contributors.
-*
-* This is free software; you can redistribute it and/or modify it
-* under the terms of the GNU Lesser General Public License as
-* published by the Free Software Foundation; either version 2.1 of
-* the License, or (at your option) any later version.
-*
-* This software is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-* Lesser General Public License for more details.
-*
-* You should have received a copy of the GNU Lesser General Public
-* License along with this software; if not, write to the Free
-* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
-*/
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
+ */
 package org.jboss.as.test.integration.domain.mixed;
 
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.ADD_CONTENT;
@@ -67,7 +50,6 @@ import org.jboss.shrinkwrap.api.exporter.ZipExporter;
 import org.jboss.shrinkwrap.api.spec.WebArchive;
 import org.junit.After;
 import org.junit.AfterClass;
-import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Test;
 
@@ -137,7 +119,7 @@ public abstract class MixedDomainDeploymentTest {
     @After
     public void confirmNoDeployments() throws Exception {
         List<ModelNode> deploymentList = getDeploymentList(PathAddress.EMPTY_ADDRESS);
-        if (deploymentList.size() > 0) {
+        if (!deploymentList.isEmpty()) {
             cleanDeployments();
         }
         deploymentList = getDeploymentList(PathAddress.EMPTY_ADDRESS);
@@ -224,8 +206,6 @@ public abstract class MixedDomainDeploymentTest {
 
     @Test
     public void testExplodedDeployment() throws Exception {
-        // TODO WFLY-9634
-        Assume.assumeFalse(supportManagedExplodedDeployment());
 
         ModelNode composite = createEmptyOperation(COMPOSITE, PathAddress.EMPTY_ADDRESS);
         ModelNode steps = composite.get(STEPS);
