@@ -1,23 +1,6 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2017, Red Hat, Inc., and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 
 package org.wildfly.extension.undertow.deployment;
@@ -25,7 +8,8 @@ package org.wildfly.extension.undertow.deployment;
 import static org.jboss.as.ee.component.Attachments.STARTUP_COUNTDOWN;
 import static org.jboss.as.server.security.SecurityMetaData.ATTACHMENT_KEY;
 import static org.jboss.as.server.security.VirtualDomainMarkerUtility.isVirtualDomainRequired;
-import static org.wildfly.extension.undertow.Capabilities.REF_LEGACY_SECURITY;
+import static org.jboss.as.web.common.VirtualHttpServerMechanismFactoryMarkerUtility.isVirtualMechanismFactoryRequired;
+import static org.wildfly.extension.undertow.logging.UndertowLogger.ROOT_LOGGER;
 
 import java.net.MalformedURLException;
 import java.time.Duration;
@@ -36,14 +20,24 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
+import java.util.OptionalInt;
 import java.util.ServiceLoader;
 import java.util.Set;
 import java.util.concurrent.Executor;
+import java.util.concurrent.ExecutorService;
 import java.util.function.BiFunction;
+import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Predicate;
+import java.util.function.Supplier;
 
-import javax.security.jacc.PolicyConfiguration;
+import jakarta.security.jacc.PolicyConfiguration;
+
+import io.undertow.servlet.api.DeploymentInfo;
+import io.undertow.servlet.api.SessionConfigWrapper;
+import io.undertow.servlet.api.SessionManagerFactory;
+import io.undertow.servlet.core.InMemorySessionManagerFactory;
 
 import org.apache.jasper.Constants;
 import org.apache.jasper.deploy.FunctionInfo;
@@ -54,17 +48,15 @@ import org.apache.jasper.deploy.TagLibraryInfo;
 import org.apache.jasper.deploy.TagLibraryValidatorInfo;
 import org.apache.jasper.deploy.TagVariableInfo;
 import org.jboss.annotation.javaee.Icon;
-import org.jboss.as.clustering.controller.CapabilityServiceConfigurator;
 import org.jboss.as.controller.PathElement;
+import org.jboss.as.controller.RequirementServiceBuilder;
+import org.jboss.as.controller.RequirementServiceTarget;
 import org.jboss.as.controller.capability.CapabilityServiceSupport;
 import org.jboss.as.ee.component.ComponentRegistry;
 import org.jboss.as.ee.component.EEModuleDescription;
 import org.jboss.as.ee.component.deployers.StartupCountdown;
 import org.jboss.as.ee.security.JaccService;
-import org.jboss.as.security.plugins.SecurityDomainContext;
-import org.jboss.as.security.service.SecurityDomainService;
 import org.jboss.as.server.ServerEnvironment;
-import org.jboss.as.server.ServerEnvironmentService;
 import org.jboss.as.server.Services;
 import org.jboss.as.server.deployment.Attachments;
 import org.jboss.as.server.deployment.DeploymentPhaseContext;
@@ -75,16 +67,19 @@ import org.jboss.as.server.deployment.DeploymentUnitProcessor;
 import org.jboss.as.server.deployment.ExplodedDeploymentMarker;
 import org.jboss.as.server.deployment.SetupAction;
 import org.jboss.as.server.deployment.module.ResourceRoot;
+import org.jboss.as.server.security.AdvancedSecurityMetaData;
 import org.jboss.as.server.security.SecurityMetaData;
 import org.jboss.as.server.suspend.SuspendController;
+import org.jboss.as.web.common.CachingWebInjectionContainer;
 import org.jboss.as.web.common.ExpressionFactoryWrapper;
 import org.jboss.as.web.common.ServletContextAttribute;
+import org.jboss.as.web.common.SimpleWebInjectionContainer;
 import org.jboss.as.web.common.WarMetaData;
 import org.jboss.as.web.common.WebComponentDescription;
 import org.jboss.as.web.common.WebInjectionContainer;
-import org.jboss.as.web.session.SessionIdentifierCodec;
 import org.jboss.as.web.session.SharedSessionManagerConfig;
 import org.jboss.dmr.ModelNode;
+import org.jboss.metadata.javaee.jboss.RunAsIdentityMetaData;
 import org.jboss.metadata.javaee.spec.DescriptionGroupMetaData;
 import org.jboss.metadata.javaee.spec.ParamValueMetaData;
 import org.jboss.metadata.web.jboss.JBossServletMetaData;
@@ -102,33 +97,32 @@ import org.jboss.msc.service.DuplicateServiceException;
 import org.jboss.msc.service.ServiceBuilder;
 import org.jboss.msc.service.ServiceController.Mode;
 import org.jboss.msc.service.ServiceName;
-import org.jboss.msc.service.ServiceTarget;
-import org.jboss.msc.value.ImmediateValue;
-import org.jboss.msc.value.InjectedValue;
 import org.jboss.vfs.VirtualFile;
 import org.wildfly.clustering.web.container.SessionManagementProvider;
 import org.wildfly.clustering.web.container.SessionManagerFactoryConfiguration;
+import org.wildfly.common.function.Functions;
 import org.wildfly.extension.io.IOServices;
 import org.wildfly.extension.requestcontroller.ControlPoint;
 import org.wildfly.extension.requestcontroller.ControlPointService;
 import org.wildfly.extension.requestcontroller.RequestControllerActivationMarker;
 import org.wildfly.extension.undertow.Capabilities;
+import org.wildfly.extension.undertow.CookieConfig;
 import org.wildfly.extension.undertow.DeploymentDefinition;
 import org.wildfly.extension.undertow.Host;
-import org.wildfly.extension.undertow.HostSingleSignOnDefinition;
 import org.wildfly.extension.undertow.ServletContainerService;
 import org.wildfly.extension.undertow.UndertowExtension;
 import org.wildfly.extension.undertow.UndertowService;
+import org.wildfly.extension.undertow.ApplicationSecurityDomainDefinition.Registration;
 import org.wildfly.extension.undertow.logging.UndertowLogger;
 import org.wildfly.extension.undertow.security.jacc.WarJACCDeployer;
 import org.wildfly.extension.undertow.session.NonDistributableSessionManagementProvider;
 import org.wildfly.extension.undertow.session.SessionManagementProviderFactory;
 import org.wildfly.security.auth.server.SecurityDomain;
+import org.wildfly.security.http.HttpServerAuthenticationMechanismFactory;
 
-import io.undertow.servlet.api.DeploymentInfo;
-import io.undertow.servlet.api.SessionManagerFactory;
-import io.undertow.servlet.core.InMemorySessionManagerFactory;
-
+/**
+ * @author <a href="mailto:ropalka@redhat.com">Richard Opalka</a>
+ */
 public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Function<SessionManagerFactoryConfiguration, SessionManagerFactory> {
 
     public static final String OLD_URI_PREFIX = "http://java.sun.com";
@@ -167,23 +161,18 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
     @Override
     public void deploy(final DeploymentPhaseContext phaseContext) throws DeploymentUnitProcessingException {
         final DeploymentUnit deploymentUnit = phaseContext.getDeploymentUnit();
+        DeploymentUnit parentDeploymentUnit = deploymentUnit.getParent();
 
+        RequirementServiceTarget serviceTarget = phaseContext.getRequirementServiceTarget();
         //install the control point for the top level deployment no matter what
-        if(RequestControllerActivationMarker.isRequestControllerEnabled(deploymentUnit)) {
-            if(deploymentUnit.getParent() == null) {
-                ControlPointService.install(phaseContext.getServiceTarget(), deploymentUnit.getName(), UndertowExtension.SUBSYSTEM_NAME);
-            }
+        if (RequestControllerActivationMarker.isRequestControllerEnabled(deploymentUnit) && parentDeploymentUnit == null) {
+            ControlPointService.install(serviceTarget, deploymentUnit.getName(), UndertowExtension.SUBSYSTEM_NAME);
         }
         final WarMetaData warMetaData = deploymentUnit.getAttachment(WarMetaData.ATTACHMENT_KEY);
         if (warMetaData == null) {
             return;
         }
-        String deploymentName;
-        if (deploymentUnit.getParent() == null) {
-            deploymentName = deploymentUnit.getName();
-        } else {
-            deploymentName = deploymentUnit.getParent().getName() + "." + deploymentUnit.getName();
-        }
+        String deploymentName = (parentDeploymentUnit != null) ? String.join(".", List.of(parentDeploymentUnit.getName(), deploymentUnit.getName())) : deploymentUnit.getName();
 
         final Map.Entry<String,String> serverHost = defaultModuleMappingProvider.getMapping(deploymentName);
         String defaultHostForDeployment;
@@ -201,28 +190,6 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
         // flag if the app is the default web module for the server/host
         boolean isDefaultWebModule = serverHost != null && serverInstanceName.equals(defaultServerForDeployment) && hostName.equals(defaultHostForDeployment);
 
-        processDeployment(warMetaData, deploymentUnit, phaseContext.getServiceTarget(), deploymentName, hostName, serverInstanceName, isDefaultWebModule);
-    }
-
-
-    private String hostNameOfDeployment(final WarMetaData metaData, String defaultHost) {
-        Collection<String> hostNames = null;
-        if (metaData.getMergedJBossWebMetaData() != null) {
-            hostNames = metaData.getMergedJBossWebMetaData().getVirtualHosts();
-        }
-        if (hostNames == null || hostNames.isEmpty()) {
-            hostNames = Collections.singleton(defaultHost);
-        }
-        String hostName = hostNames.iterator().next();
-        if (hostName == null) {
-            throw UndertowLogger.ROOT_LOGGER.nullHostName();
-        }
-        return hostName;
-    }
-
-    private void processDeployment(final WarMetaData warMetaData, final DeploymentUnit deploymentUnit, final ServiceTarget serviceTarget,
-                                   final String deploymentName, final String hostName, final String serverInstanceName, final boolean isDefaultWebModule)
-            throws DeploymentUnitProcessingException {
         ResourceRoot deploymentResourceRoot = deploymentUnit.getAttachment(Attachments.DEPLOYMENT_ROOT);
         final VirtualFile deploymentRoot = deploymentResourceRoot.getRoot();
         final Module module = deploymentUnit.getAttachment(Attachments.MODULE);
@@ -246,29 +213,19 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
                 dependentComponents.add(component);
             }
         }
-        String servletContainerName = metaData.getServletContainerName();
-        if(servletContainerName == null) {
-            servletContainerName = defaultContainer;
-        }
+        String servletContainerName = Optional.ofNullable(metaData.getServletContainerName()).orElse(this.defaultContainer);
 
-        boolean componentRegistryExists = true;
-        ComponentRegistry componentRegistry = deploymentUnit.getAttachment(org.jboss.as.ee.component.Attachments.COMPONENT_REGISTRY);
-        if (componentRegistry == null) {
-            componentRegistryExists = false;
-            //we do this to avoid lots of other null checks
-            //this will only happen if the EE subsystem is not installed
-            componentRegistry = new ComponentRegistry(null);
-        }
-
-        final WebInjectionContainer injectionContainer = new WebInjectionContainer(module.getClassLoader(), componentRegistry);
+        final boolean componentRegistryExists = deploymentUnit.getAttachment(org.jboss.as.ee.component.Attachments.COMPONENT_REGISTRY) != null;
+        final ComponentRegistry componentRegistry = componentRegistryExists ? deploymentUnit.getAttachment(org.jboss.as.ee.component.Attachments.COMPONENT_REGISTRY) : new ComponentRegistry(null);
+        final ClassLoader loader = module.getClassLoader();
+        final WebInjectionContainer injectionContainer = (metaData.getDistributable() == null) ? new CachingWebInjectionContainer(loader, componentRegistry) : new SimpleWebInjectionContainer(loader, componentRegistry);
 
         String jaccContextId = metaData.getJaccContextID();
-
         if (jaccContextId == null) {
             jaccContextId = deploymentUnit.getName();
         }
-        if (deploymentUnit.getParent() != null) {
-            jaccContextId = deploymentUnit.getParent().getName() + "!" + jaccContextId;
+        if (parentDeploymentUnit != null) {
+            jaccContextId = parentDeploymentUnit.getName() + "!" + jaccContextId;
         }
 
         String pathName = pathNameOfDeployment(deploymentUnit, metaData, isDefaultWebModule);
@@ -280,7 +237,6 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
                 additionalDependencies.addAll(dependencies);
             }
         }
-        SharedSessionManagerConfig sharedSessionManagerConfig = deploymentUnit.getParent() != null ? deploymentUnit.getParent().getAttachment(SharedSessionManagerConfig.ATTACHMENT_KEY) : null;
 
         if(!deploymentResourceRoot.isUsePhysicalCodeSource()) {
             try {
@@ -293,15 +249,7 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
         deploymentUnit.addToAttachmentList(ServletContextAttribute.ATTACHMENT_KEY, new ServletContextAttribute(Constants.PERMISSION_COLLECTION_ATTRIBUTE_NAME, deploymentUnit.getAttachment(Attachments.MODULE_PERMISSIONS)));
 
         additionalDependencies.addAll(warMetaData.getAdditionalDependencies());
-        try {
-            String capability = HostSingleSignOnDefinition.HOST_SSO_CAPABILITY.fromBaseCapability(serverInstanceName, hostName).getName();
-            capabilitySupport.getCapabilityRuntimeAPI(capability, Object.class);
-            additionalDependencies.add(capabilitySupport.getCapabilityServiceName(capability));
-        } catch (CapabilityServiceSupport.NoSuchCapabilityException e) {
-            //ignore
-        }
 
-        final ServiceName hostServiceName = UndertowService.virtualHostName(serverInstanceName, hostName);
         final ServiceName legacyDeploymentServiceName = UndertowService.deploymentServiceName(serverInstanceName, hostName, pathName);
         final ServiceName deploymentServiceName = UndertowService.deploymentServiceName(deploymentUnit.getServiceName());
 
@@ -311,8 +259,93 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
                     handler->new ComponentStartupCountdownHandler(handler, countDown));
         }
 
-        String securityDomain = deploymentUnit.getAttachment(UndertowAttachments.RESOLVED_SECURITY_DOMAIN);
+        String securityDomainName = deploymentUnit.getAttachment(UndertowAttachments.RESOLVED_SECURITY_DOMAIN);
         TldsMetaData tldsMetaData = deploymentUnit.getAttachment(TldsMetaData.ATTACHMENT_KEY);
+        final ServiceName deploymentInfoServiceName = deploymentServiceName.append(UndertowDeploymentInfoService.SERVICE_NAME);
+        final ServiceName legacyDeploymentInfoServiceName = legacyDeploymentServiceName.append(UndertowDeploymentInfoService.SERVICE_NAME);
+        final RequirementServiceBuilder<?> builder = serviceTarget.addService();
+        final Consumer<DeploymentInfo> deploymentInfo = builder.provides(deploymentInfoServiceName, legacyDeploymentInfoServiceName);
+        final Supplier<UndertowService> undertowService = builder.requires(capabilitySupport.getCapabilityServiceName(Capabilities.CAPABILITY_UNDERTOW));
+        final Supplier<ServletContainerService> servletContainerService = builder.requires(capabilitySupport.getCapabilityServiceName(Capabilities.CAPABILITY_SERVLET_CONTAINER, servletContainerName));
+        final Supplier<ComponentRegistry> componentRegistryDependency = componentRegistryExists ? builder.requires(ComponentRegistry.serviceName(deploymentUnit)) : Functions.constantSupplier(componentRegistry);
+        final Supplier<Host> host = builder.requires(Host.SERVICE_DESCRIPTOR, serverInstanceName, hostName);
+        final Supplier<SuspendController> suspendController = builder.requires(capabilitySupport.getCapabilityServiceName(Capabilities.REF_SUSPEND_CONTROLLER));
+        final Supplier<ServerEnvironment> serverEnvironment = builder.requires(ServerEnvironment.SERVICE_DESCRIPTOR);
+        Supplier<SecurityDomain> securityDomain = null;
+        Supplier<HttpServerAuthenticationMechanismFactory> mechanismFactorySupplier = null;
+        Supplier<BiFunction<DeploymentInfo, Function<String, RunAsIdentityMetaData>, Registration>> applySecurityFunction = null;
+
+        for (final ServiceName additionalDependency : additionalDependencies) {
+            builder.requires(additionalDependency);
+        }
+
+        final SecurityMetaData securityMetaData = deploymentUnit.getAttachment(ATTACHMENT_KEY);
+        if (securityDomainName != null) {
+            if (mappedSecurityDomain.test(securityDomainName)) {
+                applySecurityFunction = builder.requires(capabilitySupport.getCapabilityServiceName(Capabilities.CAPABILITY_APPLICATION_SECURITY_DOMAIN, securityDomainName));
+            } else {
+                throw ROOT_LOGGER.deploymentConfiguredForLegacySecurity();
+            }
+        }
+        else if (isVirtualDomainRequired(deploymentUnit) || isVirtualMechanismFactoryRequired(deploymentUnit)) {
+            securityDomain = builder.requires(securityMetaData.getSecurityDomain());
+
+            if (isVirtualMechanismFactoryRequired(deploymentUnit)) {
+                if (securityMetaData instanceof AdvancedSecurityMetaData) {
+                    mechanismFactorySupplier = builder.requires(((AdvancedSecurityMetaData) securityMetaData).getHttpServerAuthenticationMechanismFactory());
+                }
+            }
+        }
+
+        ROOT_LOGGER.debugf("Security domain applied is \"%s\"", securityDomainName);
+
+        Supplier<ControlPoint> controlPoint = RequestControllerActivationMarker.isRequestControllerEnabled(deploymentUnit) ? builder.requires(ControlPointService.serviceName(Optional.ofNullable(parentDeploymentUnit).orElse(deploymentUnit).getName(), UndertowExtension.SUBSYSTEM_NAME)) : null;
+
+        SharedSessionManagerConfig sharedSessionManagerConfig = parentDeploymentUnit != null ? parentDeploymentUnit.getAttachment(SharedSessionManagerConfig.ATTACHMENT_KEY) : null;
+        ServiceName sessionManagerFactoryServiceName = (sharedSessionManagerConfig != null) ? parentDeploymentUnit.getServiceName().append(SharedSessionManagerConfig.SHARED_SESSION_MANAGER_SERVICE_NAME) : deploymentServiceName.append("session");
+        ServiceName sessionConfigWrapperFactoryServiceName = (sharedSessionManagerConfig != null) ? parentDeploymentUnit.getServiceName().append(SharedSessionManagerConfig.SHARED_SESSION_AFFINITY_SERVICE_NAME) : deploymentServiceName.append("affinity");
+
+        ServletContainerService servletContainer = deploymentUnit.getAttachment(UndertowAttachments.SERVLET_CONTAINER_SERVICE);
+        Supplier<SessionManagerFactory> sessionManagerFactory = (servletContainer != null) ? builder.requires(sessionManagerFactoryServiceName) : null;
+        Supplier<Function<CookieConfig, SessionConfigWrapper>> sessionConfigWrapperFactory = (servletContainer != null) ? builder.requires(sessionConfigWrapperFactoryServiceName) : null;
+
+        if ((servletContainer != null) && (sharedSessionManagerConfig == null)) {
+            Integer maxActiveSessions = (metaData.getMaxActiveSessions() != null) ? metaData.getMaxActiveSessions() : servletContainer.getMaxSessions();
+            SessionConfigMetaData sessionConfig = metaData.getSessionConfig();
+            int defaultSessionTimeout = ((sessionConfig != null) && sessionConfig.getSessionTimeoutSet()) ? sessionConfig.getSessionTimeout() : servletContainer.getDefaultSessionTimeout();
+
+            SessionManagementProvider provider = this.getDistributableWebDeploymentProvider(phaseContext, metaData);
+            SessionManagerFactoryConfiguration configuration = new SessionManagerFactoryConfiguration() {
+                @Override
+                public String getServerName() {
+                    return serverInstanceName;
+                }
+
+                @Override
+                public String getDeploymentName() {
+                    return deploymentName;
+                }
+
+                @Override
+                public DeploymentUnit getDeploymentUnit() {
+                    return deploymentUnit;
+                }
+
+                @Override
+                public OptionalInt getMaxActiveSessions() {
+                    // Value must be positive
+                    return (maxActiveSessions != null) && (maxActiveSessions > 0) ? OptionalInt.of(maxActiveSessions) : OptionalInt.empty();
+                }
+
+                @Override
+                public Duration getDefaultSessionTimeout() {
+                    return Duration.ofMinutes(defaultSessionTimeout);
+                }
+            };
+            provider.getSessionManagerFactoryServiceInstaller(sessionManagerFactoryServiceName, configuration).install(phaseContext);
+            provider.getSessionAffinityServiceInstaller(phaseContext, sessionConfigWrapperFactoryServiceName, configuration).install(phaseContext);
+        }
+
         UndertowDeploymentInfoService undertowDeploymentInfoService = UndertowDeploymentInfoService.builder()
                 .setAttributes(deploymentUnit.getAttachmentList(ServletContextAttribute.ATTACHMENT_KEY))
                 .setContextPath(pathName)
@@ -322,7 +355,7 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
                 .setModule(module)
                 .setScisMetaData(scisMetaData)
                 .setJaccContextId(jaccContextId)
-                .setSecurityDomain(securityDomain)
+                .setSecurityDomain(securityDomainName)
                 .setTldInfo(createTldsInfo(tldsMetaData, tldsMetaData == null ? null : tldsMetaData.getSharedTlds(deploymentUnit)))
                 .setSetupActions(setupActions)
                 .setSharedSessionManagerConfig(sharedSessionManagerConfig)
@@ -339,160 +372,62 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
                 .setTempDir(warMetaData.getTempDir())
                 .setExternalResources(deploymentUnit.getAttachmentList(UndertowAttachments.EXTERNAL_RESOURCES))
                 .setAllowSuspendedRequests(deploymentUnit.getAttachmentList(UndertowAttachments.ALLOW_REQUEST_WHEN_SUSPENDED))
-                .createUndertowDeploymentInfoService();
+                .createUndertowDeploymentInfoService(deploymentInfo, undertowService, sessionManagerFactory, sessionConfigWrapperFactory,
+                        servletContainerService, componentRegistryDependency, host, controlPoint, suspendController, serverEnvironment, securityDomain, mechanismFactorySupplier, applySecurityFunction);
+        builder.setInstance(undertowDeploymentInfoService);
 
-        final ServiceName deploymentInfoServiceName = deploymentServiceName.append(UndertowDeploymentInfoService.SERVICE_NAME);
-        final ServiceName legacyDeploymentInfoServiceName = legacyDeploymentServiceName.append(UndertowDeploymentInfoService.SERVICE_NAME);
-        ServiceBuilder<DeploymentInfo> infoBuilder = serviceTarget.addService(deploymentInfoServiceName, undertowDeploymentInfoService)
-                .addAliases(legacyDeploymentInfoServiceName)
-                .addDependency(UndertowService.SERVLET_CONTAINER.append(servletContainerName), ServletContainerService.class, undertowDeploymentInfoService.getContainer())
-                .addDependency(UndertowService.UNDERTOW, UndertowService.class, undertowDeploymentInfoService.getUndertowService())
-                .addDependency(hostServiceName, Host.class, undertowDeploymentInfoService.getHost())
-                .addDependency(ServerEnvironmentService.SERVICE_NAME, ServerEnvironment.class, undertowDeploymentInfoService.getServerEnvironmentInjectedValue())
-                .addDependency(capabilitySupport.getCapabilityServiceName(Capabilities.REF_SUSPEND_CONTROLLER), SuspendController.class, undertowDeploymentInfoService.getSuspendControllerInjectedValue());
-        for (final ServiceName additionalDependency : additionalDependencies) {
-            infoBuilder.requires(additionalDependency);
-        }
-
-        if (isVirtualDomainRequired(deploymentUnit)) {
-            final SecurityMetaData securityMetaData = deploymentUnit.getAttachment(ATTACHMENT_KEY);
-            infoBuilder.addDependency(securityMetaData.getSecurityDomain(), SecurityDomain.class, undertowDeploymentInfoService.getRawSecurityDomainInjector());
-        } else if(securityDomain != null) {
-            if (mappedSecurityDomain.test(securityDomain)) {
-                infoBuilder.addDependency(
-                        deploymentUnit.getAttachment(Attachments.CAPABILITY_SERVICE_SUPPORT)
-                                .getCapabilityServiceName(
-                                        Capabilities.CAPABILITY_APPLICATION_SECURITY_DOMAIN,
-                                        securityDomain),
-                        BiFunction.class, undertowDeploymentInfoService.getSecurityFunctionInjector());
-            } else {
-                infoBuilder.addDependency(SecurityDomainService.SERVICE_NAME.append(securityDomain), SecurityDomainContext.class, undertowDeploymentInfoService.getSecurityDomainContextValue());
-            }
-        }
-
-        if(RequestControllerActivationMarker.isRequestControllerEnabled(deploymentUnit)){
-            String topLevelName;
-            if(deploymentUnit.getParent() == null) {
-                topLevelName = deploymentUnit.getName();
-            } else {
-                topLevelName = deploymentUnit.getParent().getName();
-            }
-            infoBuilder.addDependency(ControlPointService.serviceName(topLevelName, UndertowExtension.SUBSYSTEM_NAME), ControlPoint.class, undertowDeploymentInfoService.getControlPointInjectedValue());
-        }
         final Set<String> seenExecutors = new HashSet<String>();
         if (metaData.getExecutorName() != null) {
-            final InjectedValue<Executor> executor = new InjectedValue<Executor>();
-            infoBuilder.addDependency(IOServices.WORKER.append(metaData.getExecutorName()), Executor.class, executor);
+            final Supplier<Executor> executor = builder.requires(IOServices.WORKER.append(metaData.getExecutorName()));
             undertowDeploymentInfoService.addInjectedExecutor(metaData.getExecutorName(), executor);
             seenExecutors.add(metaData.getExecutorName());
         }
         if (metaData.getServlets() != null) {
             for (JBossServletMetaData servlet : metaData.getServlets()) {
                 if (servlet.getExecutorName() != null && !seenExecutors.contains(servlet.getExecutorName())) {
-                    final InjectedValue<Executor> executor = new InjectedValue<Executor>();
-                    infoBuilder.addDependency(IOServices.WORKER.append(servlet.getExecutorName()), Executor.class, executor);
+                    final Supplier<Executor> executor = builder.requires(IOServices.WORKER.append(servlet.getExecutorName()));
                     undertowDeploymentInfoService.addInjectedExecutor(servlet.getExecutorName(), executor);
                     seenExecutors.add(servlet.getExecutorName());
                 }
             }
         }
 
-        if (componentRegistryExists) {
-            infoBuilder.addDependency(ComponentRegistry.serviceName(deploymentUnit), ComponentRegistry.class, undertowDeploymentInfoService.getComponentRegistryInjectedValue());
-        } else {
-            undertowDeploymentInfoService.getComponentRegistryInjectedValue().setValue(new ImmediateValue<>(componentRegistry));
-        }
-
-        if (sharedSessionManagerConfig != null) {
-            infoBuilder.addDependency(deploymentUnit.getParent().getServiceName().append(SharedSessionManagerConfig.SHARED_SESSION_MANAGER_SERVICE_NAME), SessionManagerFactory.class, undertowDeploymentInfoService.getSessionManagerFactoryInjector());
-            infoBuilder.addDependency(deploymentUnit.getParent().getServiceName().append(SharedSessionManagerConfig.SHARED_SESSION_IDENTIFIER_CODEC_SERVICE_NAME), SessionIdentifierCodec.class, undertowDeploymentInfoService.getSessionIdentifierCodecInjector());
-        } else {
-            ServletContainerService servletContainer = deploymentUnit.getAttachment(UndertowAttachments.SERVLET_CONTAINER_SERVICE);
-            Integer maxActiveSessions = (metaData.getMaxActiveSessions() != null) ? metaData.getMaxActiveSessions() : (servletContainer != null) ? servletContainer.getMaxSessions() : null;
-            SessionConfigMetaData sessionConfig = metaData.getSessionConfig();
-            int defaultSessionTimeout = ((sessionConfig != null) && sessionConfig.getSessionTimeoutSet()) ? sessionConfig.getSessionTimeout() : (servletContainer != null) ? servletContainer.getDefaultSessionTimeout() : Integer.valueOf(30);
-            ServiceName factoryServiceName = deploymentServiceName.append("session");
-            ServiceName codecServiceName = deploymentServiceName.append("codec");
-
-            SessionManagementProvider provider = this.getDistributableWebDeploymentProvider(deploymentUnit, metaData);
-            SessionManagerFactoryConfiguration configuration = new SessionManagerFactoryConfiguration() {
-                @Override
-                public String getServerName() {
-                    return serverInstanceName;
-                }
-
-                @Override
-                public String getDeploymentName() {
-                    return deploymentName;
-                }
-
-                @Override
-                public Module getModule() {
-                    return module;
-                }
-
-                @Override
-                public Integer getMaxActiveSessions() {
-                    return maxActiveSessions;
-                }
-
-                @Override
-                public Duration getDefaultSessionTimeout() {
-                    return Duration.ofMinutes(defaultSessionTimeout);
-                }
-            };
-            CapabilityServiceConfigurator factoryConfigurator = provider.getSessionManagerFactoryServiceConfigurator(factoryServiceName, configuration);
-            CapabilityServiceConfigurator codecConfigurator = provider.getSessionIdentifierCodecServiceConfigurator(codecServiceName, configuration);
-
-            infoBuilder.addDependency(factoryConfigurator.getServiceName(), SessionManagerFactory.class, undertowDeploymentInfoService.getSessionManagerFactoryInjector());
-            infoBuilder.addDependency(codecConfigurator.getServiceName(), SessionIdentifierCodec.class, undertowDeploymentInfoService.getSessionIdentifierCodecInjector());
-
-            CapabilityServiceSupport support = deploymentUnit.getAttachment(Attachments.CAPABILITY_SERVICE_SUPPORT);
-            factoryConfigurator.configure(support).build(serviceTarget).install();
-            codecConfigurator.configure(support).build(serviceTarget).install();
-        }
-
         try {
-            infoBuilder.install();
+            builder.install();
         } catch (DuplicateServiceException e) {
             throw UndertowLogger.ROOT_LOGGER.duplicateHostContextDeployments(deploymentInfoServiceName, e.getMessage());
         }
 
-        final UndertowDeploymentService service = new UndertowDeploymentService(injectionContainer, true);
-        final ServiceBuilder<UndertowDeploymentService> builder = serviceTarget.addService(deploymentServiceName, service)
-                .addAliases(legacyDeploymentServiceName)
-                .addDependency(UndertowService.SERVLET_CONTAINER.append(defaultContainer), ServletContainerService.class, service.getContainer())
-                .addDependency(hostServiceName, Host.class, service.getHost())
-                .addDependency(deploymentInfoServiceName, DeploymentInfo.class, service.getDeploymentInfoInjectedValue());
+        final ServiceBuilder<?> udsBuilder = serviceTarget.addService(deploymentServiceName);
+        final Consumer<UndertowDeploymentService> sConsumer = udsBuilder.provides(deploymentServiceName, legacyDeploymentServiceName);
+        final Supplier<ServletContainerService> cSupplier = udsBuilder.requires(UndertowService.SERVLET_CONTAINER.append(defaultContainer));
+        final Supplier<ExecutorService> seSupplier = Services.requireServerExecutor(udsBuilder);
+        final Supplier<Host> hSupplier = udsBuilder.requires(capabilitySupport.getCapabilityServiceName(Host.SERVICE_DESCRIPTOR, serverInstanceName, hostName));
+        final Supplier<DeploymentInfo> diSupplier = udsBuilder.requires(deploymentInfoServiceName);
         for (final ServiceName webDependency : deploymentUnit.getAttachmentList(Attachments.WEB_DEPENDENCIES)) {
-            builder.requires(webDependency);
+            udsBuilder.requires(webDependency);
         }
         for (final ServiceName dependentComponent : dependentComponents) {
-            builder.requires(dependentComponent);
+            udsBuilder.requires(dependentComponent);
         }
-        // inject the server executor which can be used by the WebDeploymentService for blocking tasks in start/stop
-        // of that service
-        Services.addServerExecutorDependency(builder, service.getServerExecutorInjector());
-        builder.install();
+        udsBuilder.setInstance(new UndertowDeploymentService(sConsumer, cSupplier, seSupplier, hSupplier, diSupplier, injectionContainer, true));
+        udsBuilder.install();
 
         deploymentUnit.addToAttachmentList(Attachments.DEPLOYMENT_COMPLETE_SERVICES, deploymentServiceName);
 
         // adding Jakarta Authorization service
         final boolean elytronJacc = capabilitySupport.hasCapability(ELYTRON_JACC_CAPABILITY_NAME);
-        final boolean legacyJacc = !elytronJacc && legacySecurityInstalled(deploymentUnit);
-        if(legacyJacc || elytronJacc) {
+        if(elytronJacc) {
             WarJACCDeployer deployer = new WarJACCDeployer();
             JaccService<WarMetaData> jaccService = deployer.deploy(deploymentUnit, jaccContextId);
             if (jaccService != null) {
                 final ServiceName jaccServiceName = deploymentUnit.getServiceName().append(JaccService.SERVICE_NAME);
                 ServiceBuilder<?> jaccBuilder = serviceTarget.addService(jaccServiceName, jaccService);
-                if (deploymentUnit.getParent() != null) {
+                if (parentDeploymentUnit != null) {
                     // add dependency to parent policy
-                    final DeploymentUnit parentDU = deploymentUnit.getParent();
-                    jaccBuilder.addDependency(parentDU.getServiceName().append(JaccService.SERVICE_NAME), PolicyConfiguration.class,
-                            jaccService.getParentPolicyInjector());
+                    jaccBuilder.addDependency(parentDeploymentUnit.getServiceName().append(JaccService.SERVICE_NAME), PolicyConfiguration.class, jaccService.getParentPolicyInjector());
                 }
-                jaccBuilder.addDependency(capabilitySupport.getCapabilityServiceName(elytronJacc ? ELYTRON_JACC_CAPABILITY_NAME : LEGACY_JACC_CAPABILITY_NAME));
+                jaccBuilder.requires(capabilitySupport.getCapabilityServiceName(elytronJacc ? ELYTRON_JACC_CAPABILITY_NAME : LEGACY_JACC_CAPABILITY_NAME));
                 // add dependency to web deployment service
                 jaccBuilder.requires(deploymentServiceName);
                 jaccBuilder.setInitialMode(Mode.PASSIVE).install();
@@ -508,22 +443,31 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
         processManagement(deploymentUnit, metaData);
     }
 
-    private static boolean legacySecurityInstalled(final DeploymentUnit deploymentUnit) {
-        final CapabilityServiceSupport capabilities = deploymentUnit.getAttachment(Attachments.CAPABILITY_SERVICE_SUPPORT);
-
-        return capabilities.hasCapability(REF_LEGACY_SECURITY);
+    private static String hostNameOfDeployment(final WarMetaData metaData, String defaultHost) {
+        Collection<String> hostNames = null;
+        if (metaData.getMergedJBossWebMetaData() != null) {
+            hostNames = metaData.getMergedJBossWebMetaData().getVirtualHosts();
+        }
+        if (hostNames == null || hostNames.isEmpty()) {
+            hostNames = Collections.singleton(defaultHost);
+        }
+        String hostName = hostNames.iterator().next();
+        if (hostName == null) {
+            throw UndertowLogger.ROOT_LOGGER.nullHostName();
+        }
+        return hostName;
     }
 
     @Override
     public SessionManagerFactory apply(SessionManagerFactoryConfiguration configuration) {
-        Integer maxActiveSessions = configuration.getMaxActiveSessions();
-        return (maxActiveSessions != null) ? new InMemorySessionManagerFactory(maxActiveSessions.intValue()) : new InMemorySessionManagerFactory();
+        OptionalInt maxActiveSessions = configuration.getMaxActiveSessions();
+        return maxActiveSessions.isPresent() ? new InMemorySessionManagerFactory(maxActiveSessions.getAsInt()) : new InMemorySessionManagerFactory();
     }
 
-    private SessionManagementProvider getDistributableWebDeploymentProvider(DeploymentUnit unit, JBossWebMetaData metaData) {
+    private SessionManagementProvider getDistributableWebDeploymentProvider(DeploymentPhaseContext context, JBossWebMetaData metaData) {
         if (metaData.getDistributable() != null) {
             if (this.sessionManagementProviderFactory != null) {
-                return this.sessionManagementProviderFactory.createSessionManagementProvider(unit, metaData.getReplicationConfig());
+                return this.sessionManagementProviderFactory.createSessionManagementProvider(context, metaData.getReplicationConfig());
             }
             // Fallback to local session manager if server does not support clustering
             UndertowLogger.ROOT_LOGGER.clusteringNotSupported();
@@ -592,12 +536,10 @@ public class UndertowDeploymentProcessor implements DeploymentUnitProcessor, Fun
         }
 
         //we also register them under the new namespaces
-        for (String k : new HashSet<>(ret.keySet())) {
-            if (k != null) {
-                if (k.startsWith(OLD_URI_PREFIX)) {
-                    String newUri = k.replace(OLD_URI_PREFIX, NEW_URI_PREFIX);
-                    ret.put(newUri, ret.get(k));
-                }
+        for (Map.Entry<String, TagLibraryInfo> entry : new HashSet<>(ret.entrySet())) {
+            if (entry.getKey() != null && entry.getKey().startsWith(OLD_URI_PREFIX)) {
+                String newUri = entry.getKey().replace(OLD_URI_PREFIX, NEW_URI_PREFIX);
+                ret.put(newUri, entry.getValue());
             }
         }
 
